#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 01-Jan-2007
################################################################################
#
# Donations - Allows for automation of certain parts of the donations process
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################


use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Crypt::CBC;							# Cypher block chaining
use LWP::UserAgent;						# Library for WWW access
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handler
use Ethereal::Menu;						# Consistent Menu
use Ethereal::Param;						# Parameter control

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter hash

my %system;							# System hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect();

	# Set title
	$database->{'TITLE'} = $database->{'SYS'}{'TitDonate'};

	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Link with hash
	$database->GetHashSystem(\%system);


	# Regular display
	# Show the wares
	unless (defined($param->{'txn_id'}))
	{
		# User interface section
		DisplayDonations($database, $cgi, $param, \%system);
	}

	# Occurence of the transaction
	else
	{
		# Handle request
		HandleDonations($database, $cgi, $param, \%system);
	}


################################################################################
# Sub-Routines
################################################################################

#####################
# Display Donations
#
# This is the only interface that will be called by a regular user.  The rest
# will be handled exclusively by the PayPal interface that has been put into
# place.

sub DisplayDonations
{
	#####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $system   = shift;					# System hash

	my $login;						# Login handle
	my $menu;						# Menu handle

	my $cipher;						# Cipher handle

	my $custom;						# Custom value
	my $donations;						# Donations document


	#####################
	# Program area

	# Create instances
	$menu  = new Ethereal::Menu($database, $param);
	$login = new Ethereal::Login($database, $cgi, $param, $menu);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Initialize encryption
		$cipher = new Crypt::CBC($system{'SetKey'}, $system{'SetKeyCipher'});

		# Genetrate custom number
		$custom = join("\f", $param->{'USER'}, time);
		$custom = $cipher->encrypt_hex($custom); 

		# Retrieve body
		$donations = $database->DocumentGetDonations();
		$donations =~ s/MCUSTOM/$custom/gs;
		$donations =~ s/MINVOICE/$custom/gs;


		# Document header
		print $menu->Rewrite($database->DocumentGetHeader()), "\n";

		# Display donations section
		print $donations, "\n"; 

		# Document footer
		print $database->DocumentGetFooter(), "\n";
	}
}


#####################
# Handle Donations
#
# Will handle the requests from PayPal and jump the necessary hoops in order
# to satisfy them.   

sub HandleDonations
{
	#####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $system   = shift;					# System hash

	my $cipher;						# Cryptographic handle
	my $send;						# Mail sending

	my $key;						# Parameter key
	my $pair;						# Key/pair value
	my $query;						# Encoded query

	my $res;						# HTTP Request
	my $req;						# HTTP Request
	my $ua;							# User agent

	my $incr;						# Increments
	my $user;						# Username
	my $donation;						# Donation Email

	my @enc;						# Array of key/pairs


	#####################
	# Constants

	my $BASE = 2.50;					# Base donation
 	my $PAGE = 10.00;					# Constant donation


	#####################
	# Program area

	# Initialize encryption
	$cipher = new Crypt::CBC($system{'SetKey'}, $system{'SetKeyCipher'});

	# Encode parameters
	# Cleanup
	$param->Cleanup('XCGI', 'XDAT');

	# Generate key/pair
	foreach $key (keys(%{$param}))
	{
		# Encode key correctly
		$pair = "$key=$param->{$key}";
		$pair =~ tr/ /+/; 

		# Add to array
		push(@enc, $pair);
	}


	# Create complete string
	$query = join("&", @enc, 'cmd=_notify-validate');


	# Post back to validate
	# Handles
	$ua  = new LWP::UserAgent;
	$req = new HTTP::Request('POST', $system{'SetDonationsVerify'});
	
	# Generate request
	$req->content_type('application/x-www-form-urlencoded');
	$req->content($query);

	# Fetch results
	$res = $ua->request($req);


	# Check for reply
	# Reply failed
	if ($res->is_error) 
	{
		# HTTP error
		print "content-type: text/plain\n\nERROR\n";
	}

	# Reply success
	else
	{
		# Verified transaction
		if ($res->content =~ /VERIFIED/) 
		{
			# Evaluate
			eval
			{
				# Check the payment_status=Completed
				# Check that txn_id has not been previously processed
				# Check that receiver_email is an email address in your PayPal account

				# Completed transaction to proper location
				if (($param->{'payment_status'} =~ /completed/i)
				 && ($param->{'receiver_email'} =~ /$system->{'SetDonationsAddress'}/i)
				 && (defined($param->{'custom'})) && (length($param->{'custom'}) > 10))
				{
					# Retreive duplicate
					($res) = $database->DataGet("SELECT TransIdentifier
						FROM Transaction
						WHERE TransIdentifier=?", $param->{'txn_id'});

					# Create unique transaction
					unless (defined($res))
					{
						# Retreive values
						($user) = split(/\f/, $cipher->decrypt_hex($param->{'custom'})); 
	
						# Populate database
						$database->Write("INSERT INTO Transaction
							(TransIdentifier,
							 TransGross,
							 TransCur,
							 PuppeteerLogin)
							VALUES (?,?,?,?)",
	
						 $param->{'txn_id'},
						 $param->{'mc_gross'},
						 $param->{'mc_currency'},
						 $user
						);


						# Advanced donations
						if (($incr = int($param->{'mc_gross'} / $PAGE)) > 0)
						{
							# Database updates
							$database->Write("UPDATE Puppeteer
								SET PuppeteerVerified='yes',
								    PuppeteerName=?,
								    PuppeteerEmail=?,
								    PuppeteerHomespan=getTimestamp(PuppeteerHomeSpan) + INTERVAL '1 year'
								WHERE PuppeteerLogin=?", 
	
							 $param->{'first_name'} . ' ' . $param->{'last_name'},
							 $param->{'payer_email'},
							 $user);
						}

						# Basic donation handling
						elsif ($param->{'mc_gross'} >= $BASE)
						{
							# Flip verification tag
							$database->Write("UPDATE Puppeteer
								SET PuppeteerVerified='yes',
								    PuppeteerName=?,
								    PuppeteerEmail=?
								WHERE PuppeteerLogin=?",

							 $param->{'first_name'} . ' ' . $param->{'last_name'},
							 $param->{'payer_email'},
							 $user
							);
						}


						# Mailed thanks
						# Data retrieval
						($donation) = $database->DocumentGetDonationsMail();


						# Mail sending
						# Create instance
						$send = tie(*MAIL, 'Ethereal::Mail', $database);


						# Initial settings
						$send->SetSubject("Thank you $param->{first_name} $param->{last_name}, your contribution is appreciated ($param->{txn_id})");
						$send->SetSearch(
							'MFIRST_NAME'   => $param->{'first_name'},
							'MITEM_NAME'    => $param->{'item_name'},
							'MLAST_NAME'    => $param->{'last_name'},
							'MLOGIN_NAME'   => $user,
							'MMC_GROSS'     => $param->{'mc_gross'},
							'MMC_CURRENCY'  => $param->{'mc_currency'},
							'MPAYMENT_DATE' => $param->{'payment_date'},
							'MTXN_ID'       => $param->{'txn_id'}
						 );
						$send->ToggleWrap();

						# Recipients
						$send->AddTo("$param->{first_name} $param->{last_name} <$param->{payer_email}>");
						$send->AddBCC("$system->{SetDonationsName} <$system->{SetDonationsBCC}>");

						# Get message
						print MAIL $donation;

						# Close and send
						close(MAIL);
					}
				}
				# Commit
				$database->Commit();

				# Display message
				print "content-type: text/plain\n\nOK\n";
			};

			# Database Reverse
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();

				# Display message
				print "content-type: text/plain\n\nERROR\n";
			}
		}
	}
}
