#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 01-Jan-2007
#################################################################################
#
# Poll - Script part of Etherea Realms designed to collect and talley poll results
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Socket;							# Socket library
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Menu;						# Consistent menu
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $menu;							# Menu handle
my $param;							# Parameter handle
my $tmpl;							# Template handle

my $res;							# Results
my $statement;							# Query

my $exist;							# Does it exist
my $option;							# Option id
my $pixel = 4;							# Pixel per percent

my $name;							# Name
my $notice;							# Notice to use
my $question;							# Question

my $list;							# Generated list
my $percent;							# Percent of votes
my $spacer;							# Length of bar
my $votes;							# Total votes
my $options;							# Options

my $addr;							# IP address
my $pked;							# Packed binary address

my %poll;							# Poll hash


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);	

	# Set title
	$database->{'TITLE'} = $database->{'SYS'}{'TitPoll'};


	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();
	
	# Link with hash
	$database->GetHashPoll(\%poll);

	# Instance
	$tmpl = new Ethereal::Template(\%poll);
	$menu = new Ethereal::Menu($database, $param);


	# Document header
	print $cgi->header();
	print $menu->Rewrite($database->DocumentGetHeader()), "\n";
	

	# Poll Identifier
	# Exists
	if (defined($param->{'PID'}))
	{
		# Does the almighty concure?
		($name, $question) = $database->DataGetPoll($param->{'PID'});


		# You may pass
		if (defined($name))
		{
			# Cast your judgement
			if (defined($param->{'PRES'}))
			{
				# Pull address
				$addr = $cgi->remote_host();
				
				# Invalid IP address
				if ($addr !~ /\d{1,3}\.\d{1,3}\.\d{1,3}/)
				{
					# Retrieve address
					$pked = gethostbyname($addr);

					# Form into human readable
					$addr = inet_ntoa($pked);
				}	
				
				# Jury deliberates
				($option) = $database->DataGet("SELECT OptionID
					FROM PollOption
					WHERE PollID=?
					  AND OptionName=?",

				  $param->{'PID'},
				  $param->{'PRES'}
			  	);


				# Jury decides
				if (defined($option))
				{
					# Lousy theif
					($exist) = $database->DataGetPollVote($param->{'PID'}, $addr);


					# Wrongly accused
					unless (defined($exist))
					{
						# Create vote
						$database->Quick("INSERT INTO PollVote
							(PollID,
							 VoteHostname,
							 OptionID)
							VALUES(?,?,?)",

						 $param->{'PID'},
						 $addr,
						 $option
						);
					}
				}

				# Hung jury
				else
				{
					# Declare mis-trial
					$tmpl->Show('TmplMsg',
						MMESSAGE => $poll{'MsgInvalid'}
					 );

					# Premature exit
					exit;
				}
			}


			# Give the boy a prize
			# Notice
			$notice = (defined($param->{'PRES'})) ? $poll{'MsgVote'} : '';


			# Get total of all votes
			($votes) = $database->DataGetPollTotal($param->{'PID'});
			($options) = $database->DataGet("SELECT COUNT(*)
				FROM PollOption
				WHERE PollID=?", $param->{'PID'});


			# Tally up
			$votes = $votes - $options;

			# Avoid divide by zero
			$votes = ($votes > 0) ? $votes : 1;


			# Start display
			$database->Pull(\$statement, "SELECT
                                   COUNT(*)     AS \"Votes\",
                                   o.OptionName AS \"Name\"
                                FROM PollVote p, PollOption o
                                WHERE p.PollID=?
                                       AND p.PollID=o.PollID
                                       AND p.OptionID=o.OptionID
                                GROUP BY o.OptionId, o.OptionName
                                ORDER BY o.OptionId",

			 $param->{'PID'}
			);


			# Confirmation
			if ($res = $statement->fetchrow_hashref())
			{
				# Prep
				$list = '';

				# Loop and execute
				do
				{
					# Where are we
					$percent = int((($res->{'Votes'}-1)/$votes)*100);
					$spacer  = $percent * $pixel;

					# Display
					$list .= $tmpl->Pass('TmplResult',
						MOPTION  => $res->{'Name'},
						MPERCENT => $percent,
						MSPACER  => $spacer,
						MVOTES   => $res->{'Votes'}-1
					 );

				# Looping, looping, looping
				} while ($res = $statement->fetchrow_hashref());


				# Complete display
				$tmpl->Show('TmplPoll',
					MTITLE     => $name,
					MQUESTION  => $question,
					LSTRESULTS => $list,
					MTOTAL     => $votes,
					MNOTICE    => $notice
				 );
			}

			# End query and whining
			$statement->finish();

		}

		# Halt infidel!
		else
		{
			# Burn at the stake!
			$tmpl->Show('TmplMsg',
				MMESSAGE => $poll{'MsgMissing'}
			 );
		}
	}

	# Is missing
	else
	{
		# Signal error
		$tmpl->Show('TmplMsg', MMESSAGE=>$poll{'MsgMissing'})
	}

	# Document footer
	print $database->DocumentGetFooter(), "\n";
